"""file: chinese.py

Computes solutions of the chinese remainder systems.

"""

# need bezout from before
def bezout(a,b):
  if b == 0 : return (a,1,0)
  else :
    (d,u,v) = bezout(b,a%b)
    return (d,v,u-v*(a/b))

# suppose a prime to n; inv_mod(a,n) gives the inverse of a mod n
def inv_mod(a,n) : return bezout(a,n)[1]

"""crt(eqs) takes a list of pairs (a,n) representing the equation 'x =
a mod n' and returns the minimal solution for the system represented
by 'eqs'

"""
def crt(eqs) :
  big_prod = reduce(lambda x,y: x*y, [n for (_,n) in eqs], 1)
  part_sols = [inv_mod(big_prod/n,n)*(big_prod/n) for (_,n) in eqs]
  x = 0
  for i in range(len(eqs)): x += (part_sols[i]*eqs[i][0]) % big_prod
  return x

"""verify(eqs,s) certifies that 's' is a solution for 'eqs'

"""
def verify(eqs,sol) :
  for (a,n) in eqs:
    if a != (sol % n): return False
  return True
  
eqs = [(2,5),(4,7),(18,19),(25,46)]
s = crt(eqs)
print "A solution is", s
print "Verification:", "success" if verify(eqs,s) else "failure"
