# fib2(n) computes the nth term of the Fibonacci sequence
def fib2(n):
  f, ff =  0, 1
  for i in range(2,n): ff, f = ff+f, ff
  return ff

# fib2_better(n) follows the same idea as fib2(n) but avoid the creation
# of the list range(2,n), hence saves a little memory on large values.
def fib2_better(n):
  f, ff =  0, 1
  i = 2
  while(i<=n):
    ff, f = ff+f, ff
    i += 1
  return ff

# fib2_py(n) does the exact same things as fib2(n) and fib2_better but
# in a 'pythonic' way ; for some reason, python programmers are very
# into 'oneliner' thing like that...
#
# remark 'xrange' instead of 'range': this is a way to avoid the
# creation of a big list
def fib2_py(n):
  return reduce(lambda (f,ff),_: (ff,f+ff),xrange(2,n),(0,1))[1]


## main program

import time
# max is the maximum value ; nentry the number of test
max, nentry = 1000000, 5

for i in range(0,max,max/nentry):
  # benchmark for fib2
  start = time.clock()
  f2 = fib2(i)
  timef2 = time.clock() - start
  # benchmark for fib2
  start = time.clock()
  f2bet = fib2_better(i)
  timef2bet = time.clock() - start
  # benchmark for fib2_py
  start = time.clock()
  f2py = fib2_py(i)
  timef2py = time.clock() - start
  print ("fib(" + str(i) + ") computed in: "
         + str(timef2) +"s fib2 / "
         + str(timef2bet) +"s fib2_better / "
         + str(timef2py) +"s fib2_py")

# If you let the program run on big values, you should see that each
# function is about as good as the others.
