# fib3(n) computes the nth term of the Fibonacci sequence ; it is
# based on the following remark:
#
# fib(n) = fib(k)^2+ 2fib(k-1)fib(k) if n=2k,
#        = fib(k+1)^2+fib(k)^2 if n=2k+1

def fib3(n):
  if n == 0 or n == 1: return n
  elif n%2 == 0:
    f, ff = fib3(n/2-1), fib3(n/2)
    return ff*ff + 2*f*ff
  else:
    f, ff = fib3(n/2), fib3(n/2+1)
    return ff*ff + f*f 

# fib3 makes two recursive calls by when in fact only one is required
# ; we get around that by introducing a local function returning
# (fib(k-1),fib(k)) for k>0
def fib3_better(n):
  def aux(n): # n must be > 0
    if n == 1: return (0,1)
    else:
      (f,ff) = aux(n/2)
      if n%2 == 0: return (ff*ff + f*f, 2*f*ff + ff*ff)
      else: return (2*f*ff + ff*ff, (f+ff)*(f+ff) + ff*ff)
  return 0 if n==0 else aux(n)[1]
  
# fib2(n) only for comparison
def fib2(n):
  f, ff =  0, 1
  for i in xrange(2,n): ff, f = ff+f, ff
  return ff


## main program

import time
# max is the maximum value ; nentry the number of test
max, nentry = 1000000, 5

for i in range(0,max,max/nentry):
  # benchmark for fib3
  start = time.clock()
  f3 = fib3(i)
  timef3 = time.clock() - start
  # benchmark for fib3_better
  start = time.clock()
  f3bet = fib3_better(i)
  timef3bet = time.clock() - start
  # benchmark for fib2 for comparison
  start = time.clock()
  f2 = fib2(i)
  timef2 = time.clock() - start
  print ("fib(" + str(i) + ") computed in: "
         + str(timef3) +"s fib3 / "
         + str(timef3bet) +"s fib3_better / "
         + str(timef2) +"s fib2")

# See how fib3 is much faster than fib2 and how fib3_better is way
# faster than both. It allows us to compute very big Fibonacci number.
for i in [10**i for i in range(9)]:
  start = time.clock()
  f3bet = fib3_better(i)
  timef3bet = time.clock() - start
  print ("fib(" + str(i) + ") computed in: "
         + str(timef3bet) +"s fib3_better")

# We still get to the limit kind of fast: fib3_better(n) takes about
# 3min to compute on n=10e8 on my computer. Nothing we can really do
# about that... (In addition of the huge amount of recursive call,
# there is also the addition and multiplication of big number that
# come into play ; in fact, big number can't be dealt with as usual
# integers, because they are to big to be stored in RAM on just one
# cell ; one has to be clever and use array of number. Maybe we will
# come back to this later in the semester...
