# fact(n) computes the factorial of the integer n
def fact(n):
  factn = 1
  for i in xrange(2,n+1): factn *= i
  return factn

# Again, one can do those oneliner thingy.
def fact_py(n):
  return reduce(lambda r,x: r*x, xrange(2,n+1), 1)

# binom1(n,p) computes the number of p choices among n elements
def binom1(n,p):
  return fact(n)/(fact(p)*fact(n-p))

# binom1_better does not compute all the factorials, only the relevant
# products.
def binom1_better(n,p):
  if p < n/2: return binom1_better(n,n-p)
  num = 1
  for i in xrange(p+1,n+1): num *= i
  return num/fact(n-p)

# binom2(n) uses Pascal's recursive equation to compute the same
# result as binom1
#
# as in fib1, this is way to naive and even small coefficients as
# binom2(40,9) can not be computed in a reasonnable time.
def binom2(n,p):
  if n < p: return 0
  elif n==p or p==0: return 1 
  else: return binom2(n-1,p-1) + binom2(n-1,p)

# Still uses Pascal's equation, but stocking everything in an array to
# avoid multiple computation of the same coefficient (like in fib2)
def binom2_better(n,p):
  tr = [[1]]
  for i in xrange(1,n+1):
    line = [1]
    for j in xrange(1,i): line.append(tr[i-1][j-1]+tr[i-1][j])
    line.append(1)
    tr.append(line)
  return tr[n][p]

# But we don't actually have to keep the all array at all time. The
# previous line in the triangle is enough to compute the next one.
def binom2_best(n,p):
  prev = [1]
  for i in xrange(1,n+1):
    curr = [1]
    for j in xrange(1,i): curr.append(prev[j-1]+prev[j])
    curr.append(1)
    prev = curr
  return prev[p]

# pascal(n) prints Pascal's triangle up to the nth line
def pascal(n):
  prev = [1]
  for i in xrange(1,n+1):
    print reduce(lambda s,x: s+str(x)+"\t", prev, "")
    curr = [1]
    for j in xrange(1,i): curr.append(prev[j-1]+prev[j])
    curr.append(1)
    prev = curr
  print reduce(lambda s,x: s+str(x)+"\t", prev, "")
  return None
  
## main program

import time

# # uncomment to get benchmarks for the factorial
# max, nentry = 10**5, 5
# for i in xrange(0,max,max/nentry):
#   # benchmark for fact
#   start = time.clock()
#   f = fact(i)
#   timef = time.clock() - start
#   # benchmark for fact_py
#   start = time.clock()
#   fpy = fact_py(i)
#   timefpy = time.clock() - start
#   print ("fact(" + str(i) + ") computed in: "
#          + str(timef) +"s fact / "
#          + str(timefpy) +"s fact_py")


max, nentry = 10**3, 5
for i in xrange(0,max,max/nentry):
  for j in xrange(0,i,i/5+1):
    # benchmark for binom1_better
    start = time.clock()
    binombet = binom1_better(i,j)
    time1bet = time.clock() - start
    # benchmark for binom2_better
    start = time.clock()
    binom2bet = binom2_better(i,j)
    time2bet = time.clock() - start
    # benchmark for binom2_best
    start = time.clock()
    binom2bst = binom2_best(i,j)
    time2bst = time.clock() - start
    print ("binom(" + str((i,j)) + ") computed in: "
           + str(time1bet) +"s 1better / "
           + str(time2bet) +"s 2better / "
           + str(time2bst) +"s 2best")


# Remark here that the method exploiting Pascal's recursive formula is
# not the fastest. Keep in mind that recursivity is a powerful tool
# but sometimes direct calculation is better when it can be done
# effectively.
